require 'spec_helper'

WebMock.enable!
# WebMock.disable!

describe Sjp::Parser do
  let(:head) do
    %(
      <head>
        <meta charset='utf-8'>
      </head>
    )
  end
  let(:row) {}
  let(:table) { head + "<body><table>#{row}</table></body>" }

  context '#word=' do
    let(:text) { rand_text }
    let(:body) { text }

    # before { WebMock.enable! }
    # before { WebMock.disable! }

    it do
      stub_request(:get, "sjp.pl/#{text}").to_return(
        status: 200,
        body: body,
        headers: {}
      )
      subject.word = text
      expect(subject.word).to eq(text)
    end
  end

  # after { WebMock.disable! }
  # before { WebMock.disable! }

  context '#exist?' do
    it do
      stub_request(:get, 'sjp.pl/lorem').to_return(
        status: 200,
        body: head + '<body>
                        <p>nie występuje w słowniku</p>
                      </body>',
        headers: {}
      )
      subject.word = 'lorem'
      expect(subject).not_to be_exist
    end

    it do
      stub_request(:get, 'sjp.pl/mama').to_return(
        status: 200,
        body: head + '<body><p></p></body>',
        headers: {}
      )
      subject.word = 'mama'
      expect(subject).to be_exist
    end

    it do
      stub_request(:get, 'sjp.pl/stom').to_return(
        status: 200,
        body: head + '<body>
                        <p>słowo występuje tylko jako część innych haseł</p>
                      </body>',
        headers: {}
      )
      subject.word = 'stom'
      expect(subject).not_to be_exist
    end
  end

  describe '#allowed?' do
    let(:start) { head + '<body><p><b>' }
    let(:stop) { '</b></p></body>' }
    let(:body_allowed) { start + 'dopuszczalne w grach ' + stop }
    let(:body_not_allowed) { start + 'niedopuszczalne w grach' + stop }

    context 'true' do
      it do
        stub_request(:get, 'sjp.pl/mama').to_return(
          status: 200,
          body: body_allowed,
          headers: {}
        )
        subject.word = 'mama'
        expect(subject).to be_allowed
      end

      it do
        stub_request(:get, 'sjp.pl/a+zatem').to_return(
          status: 200,
          body: body_allowed,
          headers: {}
        )
        subject.word = 'a zatem'
        expect(subject).to be_allowed
      end

      it do
        stub_request(:get, 'sjp.pl/ta+ta+ta').to_return(
          status: 200,
          body: body_allowed,
          headers: {}
        )
        subject.word = 'ta ta ta'
        expect(subject).to be_allowed
      end
    end

    context 'false' do
      it do
        stub_request(:get, 'sjp.pl/abc').to_return(
          status: 200,
          body: body_not_allowed,
          headers: {}
        )
        subject.word = 'abc'
        expect(subject).not_to be_allowed
      end

      it do
        stub_request(:get, 'sjp.pl/mamma+mia').to_return(
          status: 200,
          body: body_not_allowed,
          headers: {}
        )
        subject.word = 'mamma mia'
        expect(subject).not_to be_allowed
      end

      it do
        stub_request(:get, 'sjp.pl/a+maiore+ad+minus').to_return(
          status: 200,
          body: body_not_allowed,
          headers: {}
        )
        subject.word = 'a maiore ad minus'
        expect(subject).not_to be_allowed
      end

      it 'stom.' do
        stub_request(:get, 'sjp.pl/stom').to_return(
          status: 200,
          body: body_not_allowed,
          headers: {}
        )
        subject.word = 'stom'
        expect(subject).not_to be_allowed
      end

      it 'a' do
        stub_request(:get, 'sjp.pl/a').to_return(
          status: 200,
          body: body_not_allowed,
          headers: {}
        )
        subject.word = 'a'
        expect(subject).not_to be_allowed
      end
    end
  end

  describe '#variety?' do
    context 'true' do
      let(:row) { '<tr><th>odmienność:</th><td>tak</td></tr>' }
      it do
        stub_request(:get, 'sjp.pl/mama').to_return(
          status: 200,
          body: table,
          headers: {}
        )
        subject.word = 'mama'
        expect(subject).to be_variety
      end
    end

    context 'false' do
      let(:row) { '<tr><th>odmienność:</th><td>nie</td></tr>' }
      it do
        stub_request(:get, 'sjp.pl/mamma+mia').to_return(
          status: 200,
          body: table,
          headers: {}
        )
        subject.word = 'mamma mia'
        expect(subject).not_to be_variety
      end
    end
  end

  # after { WebMock.disable! }
  # before { WebMock.disable! }

  context '#description' do
    let(:facebook) { '[czytaj: fejsbuk] amerykański serwis społecznościowy' }
    let(:syn) { 'potomek płci męskiej' }
    let(:corka) { 'dziecko płci żeńskiej' }

    it do
      stub_request(:get, 'sjp.pl/facebook').to_return(
        status: 200,
        body: head + "<body><p></p><p></p><p></p><p>#{facebook}</p></body>",
        headers: {}
      )
      subject.word = 'facebook'
      expect(subject.description).to eq(facebook)
    end

    it do
      stub_request(:get, 'sjp.pl/syn').to_return(
        status: 200,
        body: head + "<body><p></p><p></p><p></p><p>#{syn}</p></body>",
        headers: {}
      )
      subject.word = 'syn'
      expect(subject.description).to eq(syn)
    end

    it do
      stub_request(:get, 'sjp.pl/c%C3%B3rka').to_return(
        status: 200,
        body: head + "<body><p></p><p></p><p></p><p>#{corka}</p></body>",
        headers: {}
      )
      subject.word = 'córka'
      expect(subject.description).to eq(corka)
    end
  end
end
